/*
AIR BRAKE
CTR-style air-braking
*/

local cv_airbrake = CV_RegisterVar({
    name = "wa_airbrake",
    defaultvalue = "Off",
    flags = CV_NETVAR,
    PossibleValue = CV_OnOff
})

local airbrake_active = cv_airbrake.value

addHook("NetVars", function(net)
    airbrake_active = net(airbrake_active)
end)

addHook("MapLoad", function()
    airbrake_active = cv_airbrake.value
end)

local function loadConfig()
	local file = io.open("airbrake.cfg")
	
	if not file then return end
	
	for line in file:lines() do
		COM_BufAddText(consoleplayer, line)
	end
	
	file:close()
end

addHook("PlayerJoin", function(pnum)
	if not (consoleplayer and #consoleplayer == pnum) then return end
	
	loadConfig()
end)

local function saveConfig()
	local file, err = io.open("airbrake.cfg", "w")
	
	if not file then
		print("Couldn't open airbrake.cfg: "..err)
		return
	end
	
	file:write("wa_airbrake_waterskip "..(consoleplayer.airbrake_waterskip and "On" or "Off").."\n")
end

COM_AddCommand("wa_airbrake_waterskip", function(p, onoff)
	local possiblevalue = { on = true, yes = true, ["1"] = true }
	
	if not onoff then
		CONS_Printf(p, "wa_airbrake_waterskip - toggle airbraking when waterskipping")
		CONS_Printf(p, "Current behavior: "..(p.airbrake_waterskip and "\131allow\128 airbrake when waterskipping" or "\133disable\128 airbrake when waterskipping"))
		return
	end
	
	p.airbrake_waterskip = possiblevalue[onoff:lower()]
	
	CONS_Printf(p, "Using aibrakes when waterskipping has been "..(p.airbrake_waterskip and "\131enabled" or "\133disabled"))
	
	if p == consoleplayer then saveConfig() end
end)

freeslot("MT_TETHERDUSTTRAIL", "SPR_THDT", "S_TETHDUST1", "S_TETHDUST2", "S_FADEDUST1", "S_FADEDUST2")
states[S_TETHDUST1] = {SPR_THDT, B|FF_FULLBRIGHT, 1, nil, 0, 0, S_TETHDUST2}
states[S_TETHDUST2] = {SPR_THDT, A|FF_FULLBRIGHT|FF_ANIMATE, 4, nil, 3, 1, S_NULL}

mobjinfo[MT_TETHERDUSTTRAIL] = {
		doomednum = -1,
		spawnstate = S_TETHDUST1,    // spawnstate
		spawnhealth = 1,              // spawnhealth
		reactiontime = 8,              // reactiontime
		speed = 8,              // speed
		radius = 14*FRACUNIT,    // radius
		height = 14*FRACUNIT,    // height
		dispoffset = 0,              // display offset
		mass = 100,            // mass
		flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY|MF_SCENERY|MF_SLIDEME|MF_DONTENCOREMAP, // flags
		raisestate = S_NULL          // raisestate
}

// lua'd this from the source
// Adds gravity flipping to an object relative to its master and shifts the z coordinate accordingly.
local function K_FlipFromObject(mo, master)
	mo.eflags = (mo.eflags & ~MFE_VERTICALFLIP)|(master.eflags & MFE_VERTICALFLIP)
	mo.flags2 = (mo.flags2 & ~MF2_OBJECTFLIP)|(master.flags2 & MF2_OBJECTFLIP)

	if (mo.eflags & MFE_VERTICALFLIP)
		mo.z = ($ + master.height - FixedMul(master.scale, mo.height))
	end
end

local function K_SpawnAirBrakeDust(player, colorize) -- adjustable boost trail
	local newx --fixed_t
	local newy --fixed_t
	local ground --fixed_t
	local flame -- obj
	local travelangle -- angle

	if (not player.mo) then return end

	if ((G_BattleGametype() and player.kartstuff[k_bumper] <= 0 and player.kartstuff[k_comebacktimer])) // show sneaker boosts during a hyuu
		return
    end

	if (player.mo.eflags & MFE_VERTICALFLIP)
		ground = player.mo.ceilingz - FixedMul(mobjinfo[MT_SNEAKERTRAIL].height, player.mo.scale)
	else
		ground = player.mo.floorz
    end

	if (player.kartstuff[k_drift] != 0)
		travelangle = player.mo.angle
	else
		travelangle = R_PointToAngle2(0, 0, player.rmomx, player.rmomy)
	end
	
	for j = 0, min(3, max(0, ((player.brakeholdtime) and (( ((fade) and 15 or player.brakeholdtime) ) / (TICRATE/4)) or 0) ))
		for i = 0, 2
			local tx = P_ReturnThrustX(player.mo, travelangle, FixedMul((-12*FRACUNIT)*(j), player.mo.scale))
			local ty = P_ReturnThrustY(player.mo, travelangle, FixedMul((-12*FRACUNIT)*(j), player.mo.scale))
			local tz = player.mo.momz * j
			
			newx = player.mo.x + P_ReturnThrustX(player.mo, travelangle + ((i&1) and -1 or 1)*ANGLE_135, FixedMul((24*FRACUNIT), player.mo.scale)) + tx
			newy = player.mo.y + P_ReturnThrustY(player.mo, travelangle + ((i&1) and -1 or 1)*ANGLE_135, FixedMul((24*FRACUNIT), player.mo.scale)) + ty
			if (player.mo.standingslope)
				ground = P_GetZAt(player.mo.standingslope, newx, newy)
				if (player.mo.eflags & MFE_VERTICALFLIP)
					ground = ($ - FixedMul(mobjinfo[MT_SNEAKERTRAIL].height, player.mo.scale))
				end
			end
			flame = P_SpawnMobj(newx, newy, player.mo.z-(tz/4), MT_TETHERDUSTTRAIL)
			
			if (colorize)
				local setcolor = ColorOpposite(player.skincolor)
				flame.color = setcolor
				flame.colorized = true
			end

			flame.target = player.mo
			flame.angle = travelangle
			flame.sloperoll = player.mo.sloperoll
			flame.fuse = TICRATE*2
			flame.destscale = player.mo.scale
			P_SetScale(flame, player.mo.scale)
			// not K_MatchGenericExtraFlags so that a stolen sneaker can be seen
			K_FlipFromObject(flame, player.mo)

			flame.momx = 8

			if (not (flame and flame.valid)) then continue end
		end
	end
end

local brakedelaytime = TICRATE/3

local function midairBrake(p)
	if (not airbrake_active) then return end
	if (not p.mo) then return end
	if (p.grinding) or (p.railjumped) then return end --Air brakes don't like rails much
	if (p.mo.paragliding) then return end --Air brakes don't like gliders much
    if (p.graple and p.graple.valid) or (p.grapple and p.grapple.valid) then return end --Air brakes don't like grapple much
	if not p.airbrake_waterskip and p.kartstuff[k_waterskip] ~= 0 then
        if not jug_nowaterskip or jug_nowaterskip.isWaterSkipEnabled then return end --Air brakes don't like water skip much
    end
    
    local pks = p.kartstuff
	
	if (pks[k_spinouttimer]) then return end
	
	local pflip = (p.mo.eflags & MFE_VERTICALFLIP)
	local cantairbrake = (pks[k_spinouttimer] or p.playerstate == PST_DEAD or pks[k_respawn])
	
	if (not P_IsObjectOnGround(p.mo)) 
		if (not p.brakedelayset)
			p.brakedelay = brakedelaytime
			p.brakedelayset = true
		end
		
		if ((p.cmd.buttons & BT_BRAKE) and (not p.brakedelay) and (not cantairbrake))
			p.brakeholdtime = ((p.brakeholdtime) and ($ + 1) or 1)
			
			K_SpawnAirBrakeDust(p, false)
			
			// you instead get heavier	
			p.mo.momz = $ + (pflip and gravity or -gravity)
		end
	else
		p.brakedelayset = false
		
		p.brakeholdtime = 0
	end
	
	if (p.brakedelay)
		p.brakedelay = $ - 1
	end
end

local function airbrakepspawn(p)
	// clear out players when they spawn/respawn
	p.brakeholdtime = 0
end

addHook("PlayerThink", midairBrake)
addHook("PlayerSpawn", airbrakepspawn)