local banbehavior = { Mute = 0, PrintMuted = 1, IgnoreBan = 2, }
local cv_banbehavior = CV_RegisterVar({
    name = "chatban_behavior",
    defaultvalue = "Mute",
    possiblevalue = banbehavior,
})
local cv_wordfilter = CV_RegisterVar({
    name = "wordfilter_behavior",
    defaultvalue = "Mute",
    possiblevalue = banbehavior,
})

local wordfilter = {}
local wordfilter_local = {}

COM_AddCommand("chatban", function(p, ...)
    if not ... then
        CONS_Printf(p, "Usage: chatban <name>")
        return
    end

    local name = table.concat({...}, " ")

    for player in players.iterate do
        if player.name == name then
            player.chatban = not player.chatban

            -- :D
            if not player.chatbun and player == p then CONS_Printf(p, "// Banning yourself huh") end

            CONS_Printf(p, "Player "..name.." have been chat-"..(player.chatban and "banned" or "unbanned"))
            CONS_Printf(p, "Use same command to "..(player.chatban and "unban" or "ban").." them")
            return
        end
    end

    CONS_Printf(p, "No player with this nickname was found")
end, COM_ADMIN)

COM_AddCommand("localchatban", function(p, ...)
    if not ... then
        CONS_Printf(p, "Usage: localchatban <name>")
        return
    end

    local name = table.concat({...}, " ")

    for player in players.iterate do
        if player.name == name then
            player.localchatban = not player.localchatban

            -- :D
            if not player.localchatbun and player == p then CONS_Printf(p, "// Banning yourself huh") end

            CONS_Printf(p, "Player "..name.." have been chat-"..(player.localchatban and "banned" or "unbanned"))
            CONS_Printf(p, "Use same command to "..(player.localchatban and "unban" or "ban").." them")
            return
        end
    end

    CONS_Printf(p, "No player with this nickname was found")
end, COM_LOCAL)

COM_AddCommand("wordfilter", function(p, ...)
    if not ... then
        CONS_Printf(p, "Usage: wordfilter <word1> [word2] [word3] ...")
        return
    end

    for _, word in ipairs({...}) do
        word = word:lower()

        wordfilter[word] = not wordfilter[word]

        CONS_Printf(p, "Word "..word.." has been "..(wordfilter[word] and "added to" or "removed from").." filter")
    end
end, COM_ADMIN)

COM_AddCommand("wordfilter_local", function(p, ...)
    if not ... then
        CONS_Printf(p, "Usage: wordfilter_local <word1> [word2] [word3] ...")
        return
    end

    for _, word in ipairs({...}) do
        word = word:lower()

        wordfilter_local[word] = not wordfilter_local[word]

        CONS_Printf(p, "Word "..word.." has been "..(wordfilter_local[word] and "added to" or "removed from").." filter")
    end
end, COM_LOCAL)

local function behaviorForMsg(source, msg)
    if source == consoleplayer then return banbehavior.IgnoreBan end
    if source.chatban or source.localchatban then return cv_banbehavior.value end

    local msg = msg:lower()

    for word, _ in pairs(wordfilter) do
        if msg:find(word) ~= nil then
            return cv_wordfilter.value
        end
    end

    for word, _ in pairs(wordfilter_local) do
        if msg:find(word) ~= nil then
            return cv_wordfilter.value
        end
    end

    return banbehavior.IgnoreBan
end

addHook("PlayerMsg", function(source, type, target, msg)
    local behavior = behaviorForMsg(source, msg)

    if behavior == banbehavior.IgnoreBan then return end
    if behavior == banbehavior.Mute then return true end

    local colorchar = ''

    if SG_Color2Chat then colorchar = SG_Color2Chat[source.skincolor] end

    local msg = colorchar.."<"..source.name..">\134 [muted]"

    if type == 2 then
        msg = "\130[PM] <"..source.name..">\134 [muted]"
    end

    if target then
        chatprintf(target, msg)
    else
        chatprint(msg)
    end

    return true
end)

addHook("NetVars", function(net)
    wordfilter = net(wordfilter)
end)
